#include <stdlib.h>
#include <string.h>

#include "imago2.h"
#include "image.h"

Image::Image()
{
	w = h = psz = 0;
	pixels = 0;

	is_float = false;
}

Image::~Image()
{
	free(pixels);
}

Image::Image(const Image &image)
{
	w = image.w;
	h = image.h;
	is_float = image.is_float;
	psz = image.psz;

	if(!(pixels = malloc(w * h * psz))) {
		fprintf(stderr, "Failed to allocate pixels.\n");
		abort();
	}
	memcpy(pixels, image.pixels, w * h * psz);
}

Image &Image::operator =(const Image &image)
{
	if(&image == this)
		return *this;

	free(pixels);

	w = image.w;
	h = image.h;
	psz = image.psz;
	is_float = image.is_float;

	if(!(pixels = malloc(w * h * psz))) {
		fprintf(stderr, "Failed to allocate pixels.\n");
		abort();
	}
	memcpy(pixels, image.pixels, w * h * psz);

	return *this;
}

Image::Image(Image &&image)
{
	w = image.w;
	h = image.h;
	psz = image.psz;
	is_float = image.is_float;

	pixels = image.pixels;
	image.pixels = 0;
}

Image &Image::operator =(Image &&image)
{
	if(&image == this)
		return *this;

	free(pixels);

	w = image.w;
	h = image.h;
	psz = image.psz;
	is_float = image.is_float;

	pixels = image.pixels;
	image.pixels = 0;

	return *this;
}

bool Image::load(const char *fname)
{
	free(pixels);

	img_pixmap ipm;
	img_init(&ipm);

	if(img_load(&ipm, fname) == -1) {
		fprintf(stderr, "Failed to load image: %s.\n", fname);

		img_destroy(&ipm);
		return false;
	}

	printf("Successfully loaded image: %s\n", fname);

	img_fmt format = img_is_float(&ipm) ? IMG_FMT_RGBAF : IMG_FMT_RGBA32;
	if(img_convert(&ipm, format) == -1) {
		fprintf(stderr, "Failed to convert image %s.\n", fname);

		img_destroy(&ipm);
		return false;
	}

	w = ipm.width;
	h = ipm.height;
	psz = ipm.pixelsz;
	is_float = img_is_float(&ipm) ? true : false;

	if(!(pixels = malloc(w * h * psz))) {
		fprintf(stderr, "Failed to allocate pixels for image: %s.\n", fname);
		return false;
	}

	memcpy(pixels, ipm.pixels, w * h * psz);

	img_destroy(&ipm);
	return true;
}