#include <GL/glew.h>
#include <stdio.h>

#include "gfxapi.h"
#include "opengl/opengl.h"

extern GLFWwindow *win;
extern int win_h;
extern int win_w;

static void clear(float r, float g, float b);
static void viewport(int x, int y, int width, int height);
static void zbuffer(bool enable);
static void cull_face(Gfx_cull_face cf);

bool init_opengl()
{
	if(!glfwInit()) {
		fprintf(stderr, "Failed to initialize GLFW.\n");
		return false;
	}

	glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, 4);
	glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, 5);
	glfwWindowHint(GLFW_SRGB_CAPABLE, GLFW_TRUE);

	if(!(win = glfwCreateWindow(win_w, win_h, "glcow", 0, 0))) {
		fprintf(stderr, "Failed to create window.\n");
		return false;
	}
	glfwMakeContextCurrent(win);

	glewInit();

	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);
	glEnable(GL_FRAMEBUFFER_SRGB); // linear colorspace

	gfx_clear = clear;
	gfx_viewport = viewport;
	gfx_zbuffer = zbuffer;
	gfx_cull_face = cull_face;

	return true;
}

void cleanup_opengl()
{
	if(win) {
		glfwDestroyWindow(win);
	}
	glfwTerminate();
}

static void clear(float r, float g, float b)
{
	glClearColor(r, g, b, 1.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
}

static void viewport(int x, int y, int width, int height)
{
	glViewport(x, y, width, height);
}

static void zbuffer(bool enable)
{
	if(enable)
		glEnable(GL_DEPTH_TEST);
	else
		glDisable(GL_DEPTH_TEST);
}

static void cull_face(Gfx_cull_face cf)
{
	switch(cf) {
	case GFX_CULL_NONE:
		glDisable(GL_CULL_FACE);
		break;
	case GFX_CULL_FRONT:
		glEnable(GL_CULL_FACE);
		glCullFace(GL_FRONT);
		break;
	case GFX_CULL_BACK:
		glEnable(GL_CULL_FACE);
		glCullFace(GL_BACK);
		break;
	}
}